@@ -23,8 +23,6 @@
 # OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 # OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-from __future__ import print_function
-
 __author__ = 'eemeli.kantola@iki.fi (Eemeli Kantola)'
 
 import time
@@ -34,15 +32,7 @@ from datetime import datetime
 from asilib import ASIConnection
 from kpwrapper import SIBConnection, Triple
 
-from asibsync.util import profiled
-
-# True for production use. Setting to False makes debugging easier 
-eat_exceptions = True
-
-debug = True
-def debug_print(*strs):
-    if debug:
-        print('asi_agent: ', *strs)
+import asibsync.util as util
 
 class ASIAgent(threading.Thread):
     '''Sample usage:
@@ -50,12 +40,22 @@ class ASIAgent(threading.Thread):
     aa = ASIAgent()
     aa.set_paired_agent(sa)
     aa.start()
+
+    Usage: ASIAgent(debug = False, eat_exceptions = True)
+        debug: True for more output, False to suppress it (default).
+        eat_exceptions: True for production use (default). Setting to False makes debugging easier.
     '''
     
     POLL_INTERVAL = 5  # seconds
-    
-    def __init__(self):
+
+    def __init__(self, debug=False, eat_exceptions=True):
         super(ASIAgent, self).__init__()
+        util.bind_logging(self, debug)
+        if debug:
+            util.profile(self._poll, self._update_user)
+
+        self.eat_exceptions = eat_exceptions
+        
         self.configured_lock = threading.Lock()
         self.configured_lock.acquire()  # release after properly configured
 
@@ -70,8 +70,17 @@ class ASIAgent(threading.Thread):
         execfile(os.getenv('HOME', '.') + '/.asirc', conf)
 
         self.ac = ASIConnection(**conf['asi_user_params'])
-        self.ac.open()
-        self.uid = self.ac.session['entry']['user_id']
+        while True:  # try until we can succesfully connect
+            try:
+                self.ac.open()
+                self.uid = self.ac.session['entry']['user_id']
+                break
+            except Exception, e:
+                if self.eat_exceptions:
+                    self.log('Caught exception: %s' % e)
+                    time.sleep(ASIAgent.POLL_INTERVAL)
+                else:
+                    raise
     
     def __del__(self):
         if hasattr(self, 'ac'):  # to handle cases where an exception is raised in constructor
@@ -94,57 +103,55 @@ class ASIAgent(threading.Thread):
     def mainloop(self):
         # Ensure that we are properly configured before going on. Otherwise block here until we are.
         if self.configured_lock.acquire(False):
-            debug_print('Configuration has been done, starting.')
+            self.debug_log('Configuration has been done, starting.')
         else:
-            debug_print('This ASIAgent is not properly configured yet, waiting.')
+            self.debug_log('This ASIAgent is not properly configured yet, waiting.')
             self.configured_lock.acquire()
-            debug_print('Configuration completed, starting.')
+            self.debug_log('Configuration completed, starting.')
 
         while not self.should_stop:
             try:
                 with self.pending_sib_update_lock:
                     self._poll()
             except Exception, e:
-                if eat_exceptions:
-                    print('asi_agent: Caught exception: %s' % e)
+                if self.eat_exceptions:
+                    self.log('Caught exception: %s' % e)
                 else:
                     raise
             time.sleep(ASIAgent.POLL_INTERVAL)
         
         self.configured_lock.release()
 
-    @profiled('asi_agent')
     def _poll(self):
         user = self._get_user()
         if user.updated_at > self.asi_last_updated:
-            debug_print('Updating SIB (%s > %s)' % (user.updated_at, self.asi_last_updated))
+            self.debug_log('Updating SIB (%s > %s)' % (user.updated_at, self.asi_last_updated))
             self.pending_sib_updates = []  # ASI changes override possible SIB updates
             self.paired_agent.receive(user)
 
             self.asi_last_updated = user.updated_at
         else:
-            debug_print('No need to update SIB (%s <= %s)' % (user.updated_at, self.asi_last_updated))
+            self.debug_log('No need to update SIB (%s <= %s)' % (user.updated_at, self.asi_last_updated))
             while self.pending_sib_updates:
                 sib_update = self.pending_sib_updates.pop(0)
 
-                debug_print('Updating ASI: %s' % sib_update)
+                self.debug_log('Updating ASI: %s' % sib_update)
                 self._update_user(sib_update)
 
     def _get_user(self):
         return self.ac.get_user(self.uid)
 
-    @profiled('asi_agent')
     def _update_user(self, values):
         uid = values.pop('id')
         asi_output = self.ac.update_user(uid, **values)
         if isinstance(asi_output, dict):
-            debug_print('Updated user, got:', asi_output)
+            self.debug_log('Updated user, got:', asi_output)
             user = asi_output['entry']
             self.asi_last_updated = user['updated_at']  # prevent a useless SIB update
         else:
             raise Exception('ASI update resulted in an error:\n%s' % user)
 
     def receive(self, msg):
-        debug_print('Received', msg)
+        self.debug_log('Received', msg)
         with self.pending_sib_update_lock:
             self.pending_sib_updates.append(msg)
