@@ -23,8 +23,6 @@
 # OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 # OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-from __future__ import print_function
-
 __author__ = 'eemeli.kantola@iki.fi (Eemeli Kantola)'
 
 import collections
@@ -32,12 +30,7 @@ import uuid
 from datetime import datetime
 
 from kpwrapper import SIBConnection, Triple, uri, literal, _any
-from asibsync.util import profiled
-
-debug = True
-def debug_print(*strs):
-    if debug:
-        print('sib_agent: ', *strs)
+import asibsync.util as util
 
 def to_rdf_ontology(struct, base_uri, base_type):
     rdf = [Triple('%s#%s' % (base_uri, base_type), 'rdf:type', uri('rdfs:Class'))]
@@ -147,7 +140,11 @@ class SIBAgent:
     #                              uri('http://cos.alpha.sizl.org/people#status_message'),
     #                              None)
 
-    def __init__(self, asi_updated=None):
+    def __init__(self, asi_updated=None, debug=False):
+        util.bind_logging(self, debug)
+        if debug:
+            util.profile(self.callback, self.generate_ontology, self.subscribe, self.receive)
+        
         self.paired_agent = self.sc = None
         self.asi_updated = asi_updated
         self.sc = SIBConnection('SIB to ASI', method='preconfigured')
@@ -173,26 +170,24 @@ class SIBAgent:
             subscribe_tx.close()
         self.subscribe_txs = []
     
-    @profiled('sib_agent')
     def callback(self, added, removed):
         '''SIB subscribe callback handler. Note: removing not supported for now.'''
-        debug_print('Callback called with added=%s, removed=%s' % (added, removed))
+        self.debug_log('Callback called with added=%s, removed=%s' % (added, removed))
         
         if not self.callback_enabled:
-            debug_print('Callback is disabled')
+            self.debug_log('Callback is disabled')
             return
 
         if added:
-            debug_print('Change detected, updating ASI.')
+            self.debug_log('Change detected, updating ASI.')
             for triple in added:
                 self.paired_agent.receive(to_struct(self.sc, triple, SIBAgent.RDF_BASE_URI))
             update_done = True
         else:
-            debug_print('Change detected but no update was needed')
+            self.debug_log('Change detected but no update was needed')
 
-    @profiled('sib_agent')
     def generate_ontology(self, value):
-        debug_print('Generating ontology')
+        self.debug_log('Generating ontology')
 
         ontology = to_rdf_ontology(value, SIBAgent.RDF_BASE_URI, SIBAgent.RDF_BASE_TYPE)
         new = []
@@ -200,16 +195,15 @@ class SIBAgent:
             if not self.sc.query(t):
                 new.append(t)
         if new:
-            debug_print('Adding the missing ontology items:', new)
+            self.debug_log('Adding the missing ontology items:', new)
             self.sc.insert(new)
         else:
-            debug_print('Ontology seems to be up to date')
+            self.debug_log('Ontology seems to be up to date')
         
         return ontology
 
-    @profiled('sib_agent')
     def subscribe(self):
-        debug_print('Subscribing to changes')
+        self.debug_log('Subscribing to changes')
 
         for rdf_type in self.ontology:
             asi_obj = self.sc.query(Triple(None, 'rdf:type', rdf_type.subject))
@@ -217,26 +211,26 @@ class SIBAgent:
             query = Triple(asi_obj[0].subject, None, None)
             
             self.subscribe_txs.append(self.sc.subscribe(query, self))
-            debug_print('Subscribed to', query)
+            self.debug_log('Subscribed to', query)
 
-    @profiled('sib_agent')
     def receive(self, msg):
+        self.debug_log('receive called')
         self.callback_enabled = False
         
         if self.ontology is None:
             self.ontology = self.generate_ontology(msg)
         
-        debug_print('Received %s' % msg)
+        self.debug_log('Received %s' % msg)
         new = to_rdf_instance(msg, SIBAgent.RDF_BASE_URI, SIBAgent.RDF_BASE_TYPE, 'id')
 
         # Removing disabled for now, maybe for good
         #old_subjs = set([t.subject for t in to_rdf_instance(msg, 'id', 'http://cos.alpha.sizl.org/people/')])
         
-        #debug_print('Removing the previous triples with subjects in ', old_subjs)
+        #self.debug_log('Removing the previous triples with subjects in ', old_subjs)
         #for s in old_subjs:
         #    self.sc.remove(self.sc.query(Triple(s, None, None)))
         
-        debug_print('Inserting', new)
+        self.debug_log('Inserting', new)
         self.sc.insert(new)
 
         # Subscribe to changes to the data that has been synced
